@@ -8,7 +8,7 @@ use integer;
 use Pod::Escapes 1.03 ();
 use Pod::Simple::LinkSection ();
 use Pod::Simple::BlackBox ();
-use utf8;
+#use utf8;
 
 use vars qw(
   $VERSION @ISA
@@ -18,7 +18,7 @@ use vars qw(
 );
 
 @ISA = ('Pod::Simple::BlackBox');
-$VERSION = '2.04';
+$VERSION = '2.06';
 
 @Known_formatting_codes = qw(I B C L E F S X Z); 
 %Known_formatting_codes = map(($_=>1), @Known_formatting_codes);
@@ -39,9 +39,13 @@ BEGIN {
   unless(MANY_LINES() >= 1) {
     die "MANY_LINES is too small (", MANY_LINES(), ")!\nAborting";
   }
+  if(defined &UNICODE) { }
+  elsif($] >= 5.008)   { *UNICODE = sub() {1}  }
+  else                 { *UNICODE = sub() {''} }
 }
 if(DEBUG > 2) {
   print "# We are ", ASCII ? '' : 'not ', "in ASCII-land\n";
+  print "# We are under a Unicode-safe Perl.\n";
 }
 
 # Design note:
@@ -76,6 +80,8 @@ __PACKAGE__->_accessorize(
   'bare_output',       # For some subclasses: whether to prepend
                        #  header-code and postpend footer-code
 
+  'fullstop_space_harden', # Whether to turn ".  " into ".[nbsp] ";
+
   'nix_X_codes',       # whether to ignore X<...> codes
   'merge_text',        # whether to avoid breaking a single piece of
                        #  text up into several events
@@ -83,6 +89,8 @@ __PACKAGE__->_accessorize(
  'content_seen',      # whether we've seen any real Pod content
  'errors_seen',       # TODO: document.  whether we've seen any errors (fatal or not)
 
+ 'codes_in_verbatim', # for PseudoPod extensions
+
  'code_handler',      # coderef to call when a code (non-pod) line is seen
  'cut_handler',       # coderef to call when a =cut line is seen
  #Called like:
@@ -135,9 +143,17 @@ sub output_string {
   $$x = '' unless defined $$x;
   DEBUG > 4 and print "# Output string set to $x ($$x)\n";
   $this->{'output_fh'} = Pod::Simple::TiedOutFH->handle_on($_[0]);
-  return $this->{'output_string'} = ${ $this->{'output_fh'} };
+  return
+    $this->{'output_string'} = $_[0];
+    #${ ${ $this->{'output_fh'} } };
 }
 
+sub abandon_output_string { $_[0]->abandon_output_fh; delete $_[0]{'output_string'} }
+sub abandon_output_fh     { $_[0]->output_fh(undef) }
+# These don't delete the string or close the FH -- they just delete our
+#  references to it/them.
+# TODO: document these
+
 #@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 
 sub new {
@@ -1004,7 +1020,9 @@ sub _treat_Ls {  # Process our dear dear friends, the L<...> sequences
       ) {
         $treelet->[$i][1]{'type'} = 'url';
         $treelet->[$i][1]{'content-implicit'} = 'yes';
-        
+
+        # TODO: deal with rel: URLs here?
+
         if( 3 == @{ $treelet->[$i] } ) {
           # But if it IS just one text node (most common case)
           DEBUG > 1 and printf qq{Catching "%s as " as ho-hum L<URL> link.\n},
@@ -1236,7 +1254,7 @@ sub _treat_Ls {  # Process our dear dear friends, the L<...> sequences
 sub _treat_Es {
   my($self,@stack) = @_;
 
-  my($i, $treelet, $content, $replacer);
+  my($i, $treelet, $content, $replacer, $charnum);
   my $start_line = $stack[0][1]{'start_line'};
 
   # A recursive algorithm implemented iteratively!  Whee!
@@ -1295,12 +1313,21 @@ sub _treat_Es {
 
       DEBUG > 1 and print "Ogling E<$content>\n";
 
-      if(defined( $replacer = Pod::Escapes::e2char($content) )) {
-        DEBUG > 1 and print "Replacing E<$content> with $replacer\n";
-      } else {
-        DEBUG > 1 and print "I don't know how to deal with E<$content>\n";
+      $charnum  = Pod::Escapes::e2charnum($content);
+      DEBUG > 1 and print " Considering E<$content> with char ",
+        defined($charnum) ? $charnum : "undef", ".\n";
+
+      if(!defined( $charnum )) {
+        DEBUG > 1 and print "I don't know how to deal with E<$content>.\n";
         $self->whine( $start_line, "Unknown E content in E<$content>" );
         $replacer = "E<$content>"; # better than nothing
+      } elsif($charnum >= 255 and !UNICODE) {
+        $replacer = ASCII ? "\xA4" : "?";
+        DEBUG > 1 and print "This Perl version can't handle ", 
+          "E<$content> (chr $charnum), so replacing with $replacer\n";
+      } else {
+        $replacer = Pod::Escapes::e2char($content);
+        DEBUG > 1 and print " Replacing E<$content> with $replacer\n";
       }
 
       splice(@$treelet, $i, 1, $replacer); # no need to back up $i, tho
