@@ -20,7 +20,6 @@ package Pod::Simple::BlackBox;
 # Every node in a treelet is a ['name', {attrhash}, ...children...]
 
 use integer; # vroom!
-use utf8;
 use strict;
 use Carp ();
 BEGIN {
@@ -526,237 +525,17 @@ sub _ponder_paragraph_buffer {
     DEBUG > 1 and print "Pondering a $para_type paragraph, given the stack: (",
       $self->_dump_curr_open(), ")\n";
     
-    if($para_type eq '=for') { #//////////////////////////////////////////////
-      # Fake it out as a begin/end
-      my $target;
+    if($para_type eq '=for') {
+      next if $self->_ponder_for($para,$curr_open,$paras);
 
-      if(grep $_->[1]{'~ignore'}, @$curr_open) {
-        DEBUG > 1 and print "Ignoring ignorable =for\n";
-        next;
-      }
+    } elsif($para_type eq '=begin') {
+      next if $self->_ponder_begin($para,$curr_open,$paras);
 
-      for(my $i = 2; $i < @$para; ++$i) {
-        if($para->[$i] =~ s/^\s*(\S+)\s*//s) {
-          $target = $1;
-          last;
-        }
-      }
-      unless(defined $target) {
-        $self->whine(
-          $para->[1]{'start_line'},
-          "=for without a target?"
-        );
-        next;
-      }
-      DEBUG > 1 and
-       print "Faking out a =for $target as a =begin $target / =end $target\n";
-      
-      $para->[0] = 'Data';
-      
-      unshift @$paras,
-        ['=begin',
-          {'start_line' => $para->[1]{'start_line'}, '~really' => '=for'},
-          $target,
-        ],
-        $para,
-        ['=end',
-          {'start_line' => $para->[1]{'start_line'}, '~really' => '=for'},
-          $target,
-        ],
-      ;
-      
-      next;
-      
-    } elsif($para_type eq '=begin') { #///////////////////////////////////////
+    } elsif($para_type eq '=end') {
+      next if $self->_ponder_end($para,$curr_open,$paras);
 
-      my $content = join ' ', splice @$para, 2;
-      $content =~ s/^\s+//s;
-      $content =~ s/\s+$//s;
-      unless(length($content)) {
-        $self->whine(
-          $para->[1]{'start_line'},
-          "=begin without a target?"
-        );
-        DEBUG and print "Ignoring targetless =begin\n";
-        next;
-      }
-      
-      unless($content =~ m/^\S+$/s) {  # i.e., unless it's one word
-        $self->whine(
-          $para->[1]{'start_line'},
-          "'=begin' only takes one parameter, not several as in '=begin $content'"
-        );
-        DEBUG and print "Ignoring unintelligible =begin $content\n";
-        next;
-      }
-
-
-      $para->[1]{'target'} = $content;  # without any ':'
-
-      $content =~ s/^:!/!:/s;
-      my $neg;  # whether this is a negation-match
-      $neg = 1        if $content =~ s/^!//s;
-      my $to_resolve;  # whether to process formatting codes
-      $to_resolve = 1 if $content =~ s/^://s;
-      
-      my $dont_ignore; # whether this target matches us
-      
-      foreach my $target_name (
-        split(',', $content, -1),
-        $neg ? () : '*'
-      ) {
-        DEBUG > 2 and
-         print " Considering whether =begin $content matches $target_name\n";
-        next unless $self->{'accept_targets'}{$target_name};
-        
-        DEBUG > 2 and
-         print "  It DOES match the acceptable target $target_name!\n";
-        $to_resolve = 1
-          if $self->{'accept_targets'}{$target_name} eq 'force_resolve';
-        $dont_ignore = 1;
-        $para->[1]{'target_matching'} = $target_name;
-        last; # stop looking at other target names
-      }
-
-      if($neg) {
-        if( $dont_ignore ) {
-          $dont_ignore = '';
-          delete $para->[1]{'target_matching'};
-          DEBUG > 2 and print " But the leading ! means that this is a NON-match!\n";
-        } else {
-          $dont_ignore = 1;
-          $para->[1]{'target_matching'} = '!';
-          DEBUG > 2 and print " But the leading ! means that this IS a match!\n";
-        }
-      }
-
-      $para->[0] = '=for';  # Just what we happen to call these, internally
-      $para->[1]{'~really'} ||= '=begin';
-      $para->[1]{'~ignore'}   = (! $dont_ignore) || 0;
-      $para->[1]{'~resolve'}  = $to_resolve || 0;
-
-      DEBUG > 1 and print " Making note to ", $dont_ignore ? 'not ' : '',
-        "ignore contents of this region\n";
-      DEBUG > 1 and $dont_ignore and print " Making note to treat contents as ",
-        ($to_resolve ? 'verbatim/plain' : 'data'), " paragraphs\n";
-      DEBUG > 1 and print " (Stack now: ", $self->_dump_curr_open(), ")\n";
-
-      push @$curr_open, $para;
-      if(!$dont_ignore or scalar grep $_->[1]{'~ignore'}, @$curr_open) {
-        DEBUG > 1 and print "Ignoring ignorable =begin\n";
-      } else {
-        $self->{'content_seen'} ||= 1;
-        $self->_handle_element_start(($scratch='for'), $para->[1]);
-      }
-
-      next;
-      
-    } elsif($para_type eq '=end') { #/////////////////////////////////////////
-
-      my $content = join ' ', splice @$para, 2;
-      $content =~ s/^\s+//s;
-      $content =~ s/\s+$//s;
-      DEBUG and print "Ogling '=end $content' directive\n";
-      
-      unless(length($content)) {
-        $self->whine(
-          $para->[1]{'start_line'},
-          "'=end' without a target?" . (
-            ( @$curr_open and $curr_open->[-1][0] eq '=for' )
-            ? ( " (Should be \"=end " . $curr_open->[-1][1]{'target'} . '")' )
-            : ''
-          )
-        );
-        DEBUG and print "Ignoring targetless =end\n";
-        next;
-      }
-      
-      unless($content =~ m/^\S+$/) {  # i.e., unless it's one word
-        $self->whine(
-          $para->[1]{'start_line'},
-          "'=end $content' is invalid.  (Stack: "
-          . $self->_dump_curr_open() . ')'
-        );
-        DEBUG and print "Ignoring mistargetted =end $content\n";
-        next;
-      }
-      
-      unless(@$curr_open and $curr_open->[-1][0] eq '=for') {
-        $self->whine(
-          $para->[1]{'start_line'},
-          "=end $content without matching =begin.  (Stack: "
-          . $self->_dump_curr_open() . ')'
-        );
-        DEBUG and print "Ignoring mistargetted =end $content\n";
-        next;
-      }
-      
-      unless($content eq $curr_open->[-1][1]{'target'}) {
-        $self->whine(
-          $para->[1]{'start_line'},
-          "=end $content doesn't match =begin " 
-          . $curr_open->[-1][1]{'target'}
-          . ".  (Stack: "
-          . $self->_dump_curr_open() . ')'
-        );
-        DEBUG and print "Ignoring mistargetted =end $content at line $para->[1]{'start_line'}\n";
-        next;
-      }
-
-      # Else it's okay to close...
-      if(grep $_->[1]{'~ignore'}, @$curr_open) {
-        DEBUG > 1 and print "Not firing any event for this =end $content because in an ignored region\n";
-        # And that may be because of this to-be-closed =for region, or some
-        #  other one, but it doesn't matter.
-      } else {
-        $curr_open->[-1][1]{'start_line'} = $para->[1]{'start_line'};
-          # what's that for?
-        
-        $self->{'content_seen'} ||= 1;
-        $self->_handle_element_end( $scratch = 'for' );
-      }
-      DEBUG > 1 and print "Popping $curr_open->[-1][0] $curr_open->[-1][1]{'target'} because of =end $content\n";
-      pop @$curr_open;
-
-      next;
-      
-    } elsif($para_type eq '~end') { #/////////////////////////////////////////
-      # The virtual end-document signal
-      
-      if(@$curr_open) { # Deal with things left open
-        DEBUG and print "Stack is nonempty at end-document: (",
-          $self->_dump_curr_open(), ")\n";
-          
-        DEBUG > 9 and print "Stack: ", pretty($curr_open), "\n";
-        unshift @$paras, $self->_closers_for_all_curr_open;
-        # Make sure there is exactly one ~end in the parastack, at the end:
-        @$paras = grep $_->[0] ne '~end', @$paras;
-        push @$paras, $para, $para;
-         # We need two -- once for the next cycle where we
-         #  generate errata, and then another to be at the end
-         #  when that loop back around to process the errata.
-        next;
-        
-      } else {
-        DEBUG and print "Okay, stack is empty now.\n";
-      }
-      
-      # Try generating errata section, if applicable
-      unless($self->{'~tried_gen_errata'}) {
-        $self->{'~tried_gen_errata'} = 1;
-        my @extras = $self->_gen_errata();
-        if(@extras) {
-          unshift @$paras, @extras;
-          DEBUG and print "Generated errata... relooping...\n";
-          next;  # I.e., loop around again to process these fake-o paragraphs
-        }
-      }
-      
-      splice @$paras; # Well, that's that for this paragraph buffer.
-      DEBUG and print "Throwing end-document event.\n";
-
-      $self->_handle_element_end( $scratch = 'Document' );
-      next; # Hasta la byebye
+    } elsif($para_type eq '~end') { # The virtual end-document signal
+      next if $self->_ponder_doc_end($para,$curr_open,$paras);
     }
 
 
@@ -770,97 +549,17 @@ sub _ponder_paragraph_buffer {
     #~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
     # ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
 
-    if($para_type eq '=pod') { #//////////////////////////////////////////////
-      $self->whine(
-        $para->[1]{'start_line'},
-        "=pod directives shouldn't be over one line long!  Ignoring all "
-         . (@$para - 2) . " lines of content"
-      ) if @$para > 3;
-      # Content is always ignored.
-      
+    if($para_type eq '=pod') {
+      $self->_ponder_pod($para,$curr_open,$paras);
 
-    } elsif($para_type eq '=over') { #////////////////////////////////////////
-      next unless @$paras;
-      my $list_type;
+    } elsif($para_type eq '=over') {
+      next if $self->_ponder_over($para,$curr_open,$paras);
 
-      if($paras->[0][0] eq '=item') { # most common case
-        $list_type = $self->_get_initial_item_type($paras->[0]);
+    } elsif($para_type eq '=back') {
+      next if $self->_ponder_back($para,$curr_open,$paras);
 
-      } elsif($paras->[0][0] eq '=back') {
-        # Ignore empty lists.  TODO: make this an option?
-        shift @$paras;
-        next;
-        
-      } elsif($paras->[0][0] eq '~end') {
-        $self->whine(
-          $para->[1]{'start_line'},
-          "=over is the last thing in the document?!"
-        );
-        next; # But feh, ignore it.
-      } else {
-        $list_type = 'block';
-      }
-      $para->[1]{'~type'} = $list_type;
-      push @$curr_open, $para;
-       # yes, we reuse the paragraph as a stack item
-      
-      my $content = join ' ', splice @$para, 2;
-      my $overness;
-      if($content =~ m/^\s*$/s) {
-        $para->[1]{'indent'} = 4;
-      } elsif($content =~ m/^\s*((?:\d*\.)?\d+)\s*$/s) {
-        no integer;
-        $para->[1]{'indent'} = $1;
-        if($1 == 0) {
-          $self->whine(
-            $para->[1]{'start_line'},
-            "Can't have a 0 in =over $content"
-          );
-          $para->[1]{'indent'} = 4;
-        }
-      } else {
-        $self->whine(
-          $para->[1]{'start_line'},
-          "=over should be: '=over' or '=over positive_number'"
-        );
-        $para->[1]{'indent'} = 4;
-      }
-      DEBUG > 1 and print "=over found of type $list_type\n";
-      
-      $self->{'content_seen'} ||= 1;
-      $self->_handle_element_start(($scratch = 'over-' . $list_type), $para->[1]);
-      
-    } elsif($para_type eq '=back') { #////////////////////////////////////////
+    } else {
 
-      # TODO: fire off </item-number> or </item-bullet> or </item-text> ??
-
-      my $content = join ' ', splice @$para, 2;
-      if($content =~ m/\S/) {
-        $self->whine(
-          $para->[1]{'start_line'},
-          "=back doesn't take any parameters, but you said =back $content"
-        );
-      }
-
-      if(@$curr_open and $curr_open->[-1][0] eq '=over') {
-        DEBUG > 1 and print "=back happily closes matching =over\n";
-        # Expected case: we're closing the most recently opened thing
-        #my $over = pop @$curr_open;
-        $self->{'content_seen'} ||= 1;
-        $self->_handle_element_end( $scratch =
-          'over-' . ( (pop @$curr_open)->[1]{'~type'} )
-        );
-      } else {
-        DEBUG > 1 and print "=back found without a matching =over.  Stack: (",
-            join(', ', map $_->[0], @$curr_open), ").\n";
-        $self->whine(
-          $para->[1]{'start_line'},
-          '=back without =over'
-        );
-        next; # and ignore it
-      }
-      
-    } else { #////////////////////////////////////////////////////////////////
       # All non-magical codes!!!
       
       # Here we start using $para_type for our own twisted purposes, to
@@ -1124,54 +823,11 @@ sub _ponder_paragraph_buffer {
 
       #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
       if($para_type eq 'Plain') {
-        DEBUG and print " giving plain treatment...\n";
-        unless( @$para == 2 or ( @$para == 3 and $para->[2] eq '' )
-          or $para->[1]{'~cooked'}
-        ) {
-          push @$para,
-          @{$self->_make_treelet(
-            join("\n", splice(@$para, 2)),
-            $para->[1]{'start_line'}
-          )};
-        }
-        # Empty paragraphs don't need a treelet for any reason I can see.
-        # And precooked paragraphs already have a treelet.
-        
+        $self->_ponder_Plain($para);
       } elsif($para_type eq 'Verbatim') {
-        DEBUG and print " giving verbatim treatment...\n";
-      
-        $para->[1]{'xml:space'} = 'preserve';
-        for($i = 2; $i < @$para; $i++) {
-          foreach my $line ($para->[$i]) { # just for aliasing
-            while( $line =~
-              # Sort of adapted from Text::Tabs -- yes, it's hardwired in that
-              # tabs are at every EIGHTH column.  For portability, it has to be
-              # one setting everywhere, and 8th wins.
-              s/^([^\t]*)(\t+)/$1.(" " x ((length($2)<<3)-(length($1)&7)))/e
-            ) {}
-
-            # TODO: whinge about (or otherwise treat) unindented or overlong lines
-
-          }
-        }
-        
-        # Now the VerbatimFormatted hoodoo...
-        if( $self->{'accept_codes'} and
-            $self->{'accept_codes'}{'VerbatimFormatted'}
-        ) {
-          while(@$para > 3 and $para->[-1] !~ m/\S/) { pop @$para }
-           # Kill any number of terminal newlines
-          $self->_verbatim_format($para);
-        } else {
-          push @$para, join "\n", splice(@$para, 2) if @$para > 3;
-          $para->[-1] =~ s/\n+$//s; # Kill any number of terminal newlines
-        }
-        
+        $self->_ponder_Verbatim($para);        
       } elsif($para_type eq 'Data') {
-        DEBUG and print " giving data treatment...\n";
-        $para->[1]{'xml:space'} = 'preserve';
-        push @$para, join "\n", splice(@$para, 2) if @$para > 3;
-        
+        $self->_ponder_Data($para);
       } else {
         die "\$para type is $para_type -- how did that happen?";
         # Shouldn't happen.
@@ -1191,6 +847,583 @@ sub _ponder_paragraph_buffer {
   return;
 }
 
+###########################################################################
+# The sub-ponderers...
+
+
+
+sub _ponder_for {
+  my ($self,$para,$curr_open,$paras) = @_;
+
+  # Fake it out as a begin/end
+  my $target;
+
+  if(grep $_->[1]{'~ignore'}, @$curr_open) {
+    DEBUG > 1 and print "Ignoring ignorable =for\n";
+    return 1;
+  }
+
+  for(my $i = 2; $i < @$para; ++$i) {
+    if($para->[$i] =~ s/^\s*(\S+)\s*//s) {
+      $target = $1;
+      last;
+    }
+  }
+  unless(defined $target) {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "=for without a target?"
+    );
+    return 1;
+  }
+  DEBUG > 1 and
+   print "Faking out a =for $target as a =begin $target / =end $target\n";
+  
+  $para->[0] = 'Data';
+  
+  unshift @$paras,
+    ['=begin',
+      {'start_line' => $para->[1]{'start_line'}, '~really' => '=for'},
+      $target,
+    ],
+    $para,
+    ['=end',
+      {'start_line' => $para->[1]{'start_line'}, '~really' => '=for'},
+      $target,
+    ],
+  ;
+  
+  return 1;
+}
+
+sub _ponder_begin {
+  my ($self,$para,$curr_open,$paras) = @_;
+  my $content = join ' ', splice @$para, 2;
+  $content =~ s/^\s+//s;
+  $content =~ s/\s+$//s;
+  unless(length($content)) {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "=begin without a target?"
+    );
+    DEBUG and print "Ignoring targetless =begin\n";
+    return 1;
+  }
+  
+  unless($content =~ m/^\S+$/s) {  # i.e., unless it's one word
+    $self->whine(
+      $para->[1]{'start_line'},
+      "'=begin' only takes one parameter, not several as in '=begin $content'"
+    );
+    DEBUG and print "Ignoring unintelligible =begin $content\n";
+    return 1;
+  }
+
+
+  $para->[1]{'target'} = $content;  # without any ':'
+
+  $content =~ s/^:!/!:/s;
+  my $neg;  # whether this is a negation-match
+  $neg = 1        if $content =~ s/^!//s;
+  my $to_resolve;  # whether to process formatting codes
+  $to_resolve = 1 if $content =~ s/^://s;
+  
+  my $dont_ignore; # whether this target matches us
+  
+  foreach my $target_name (
+    split(',', $content, -1),
+    $neg ? () : '*'
+  ) {
+    DEBUG > 2 and
+     print " Considering whether =begin $content matches $target_name\n";
+    next unless $self->{'accept_targets'}{$target_name};
+    
+    DEBUG > 2 and
+     print "  It DOES match the acceptable target $target_name!\n";
+    $to_resolve = 1
+      if $self->{'accept_targets'}{$target_name} eq 'force_resolve';
+    $dont_ignore = 1;
+    $para->[1]{'target_matching'} = $target_name;
+    last; # stop looking at other target names
+  }
+
+  if($neg) {
+    if( $dont_ignore ) {
+      $dont_ignore = '';
+      delete $para->[1]{'target_matching'};
+      DEBUG > 2 and print " But the leading ! means that this is a NON-match!\n";
+    } else {
+      $dont_ignore = 1;
+      $para->[1]{'target_matching'} = '!';
+      DEBUG > 2 and print " But the leading ! means that this IS a match!\n";
+    }
+  }
+
+  $para->[0] = '=for';  # Just what we happen to call these, internally
+  $para->[1]{'~really'} ||= '=begin';
+  $para->[1]{'~ignore'}   = (! $dont_ignore) || 0;
+  $para->[1]{'~resolve'}  = $to_resolve || 0;
+
+  DEBUG > 1 and print " Making note to ", $dont_ignore ? 'not ' : '',
+    "ignore contents of this region\n";
+  DEBUG > 1 and $dont_ignore and print " Making note to treat contents as ",
+    ($to_resolve ? 'verbatim/plain' : 'data'), " paragraphs\n";
+  DEBUG > 1 and print " (Stack now: ", $self->_dump_curr_open(), ")\n";
+
+  push @$curr_open, $para;
+  if(!$dont_ignore or scalar grep $_->[1]{'~ignore'}, @$curr_open) {
+    DEBUG > 1 and print "Ignoring ignorable =begin\n";
+  } else {
+    $self->{'content_seen'} ||= 1;
+    $self->_handle_element_start((my $scratch='for'), $para->[1]);
+  }
+
+  return 1;
+}
+
+sub _ponder_end {
+  my ($self,$para,$curr_open,$paras) = @_;
+  my $content = join ' ', splice @$para, 2;
+  $content =~ s/^\s+//s;
+  $content =~ s/\s+$//s;
+  DEBUG and print "Ogling '=end $content' directive\n";
+  
+  unless(length($content)) {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "'=end' without a target?" . (
+        ( @$curr_open and $curr_open->[-1][0] eq '=for' )
+        ? ( " (Should be \"=end " . $curr_open->[-1][1]{'target'} . '")' )
+        : ''
+      )
+    );
+    DEBUG and print "Ignoring targetless =end\n";
+    return 1;
+  }
+  
+  unless($content =~ m/^\S+$/) {  # i.e., unless it's one word
+    $self->whine(
+      $para->[1]{'start_line'},
+      "'=end $content' is invalid.  (Stack: "
+      . $self->_dump_curr_open() . ')'
+    );
+    DEBUG and print "Ignoring mistargetted =end $content\n";
+    return 1;
+  }
+  
+  unless(@$curr_open and $curr_open->[-1][0] eq '=for') {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "=end $content without matching =begin.  (Stack: "
+      . $self->_dump_curr_open() . ')'
+    );
+    DEBUG and print "Ignoring mistargetted =end $content\n";
+    return 1;
+  }
+  
+  unless($content eq $curr_open->[-1][1]{'target'}) {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "=end $content doesn't match =begin " 
+      . $curr_open->[-1][1]{'target'}
+      . ".  (Stack: "
+      . $self->_dump_curr_open() . ')'
+    );
+    DEBUG and print "Ignoring mistargetted =end $content at line $para->[1]{'start_line'}\n";
+    return 1;
+  }
+
+  # Else it's okay to close...
+  if(grep $_->[1]{'~ignore'}, @$curr_open) {
+    DEBUG > 1 and print "Not firing any event for this =end $content because in an ignored region\n";
+    # And that may be because of this to-be-closed =for region, or some
+    #  other one, but it doesn't matter.
+  } else {
+    $curr_open->[-1][1]{'start_line'} = $para->[1]{'start_line'};
+      # what's that for?
+    
+    $self->{'content_seen'} ||= 1;
+    $self->_handle_element_end( my $scratch = 'for' );
+  }
+  DEBUG > 1 and print "Popping $curr_open->[-1][0] $curr_open->[-1][1]{'target'} because of =end $content\n";
+  pop @$curr_open;
+
+  return 1;
+} 
+
+sub _ponder_doc_end {
+  my ($self,$para,$curr_open,$paras) = @_;
+  if(@$curr_open) { # Deal with things left open
+    DEBUG and print "Stack is nonempty at end-document: (",
+      $self->_dump_curr_open(), ")\n";
+      
+    DEBUG > 9 and print "Stack: ", pretty($curr_open), "\n";
+    unshift @$paras, $self->_closers_for_all_curr_open;
+    # Make sure there is exactly one ~end in the parastack, at the end:
+    @$paras = grep $_->[0] ne '~end', @$paras;
+    push @$paras, $para, $para;
+     # We need two -- once for the next cycle where we
+     #  generate errata, and then another to be at the end
+     #  when that loop back around to process the errata.
+    return 1;
+    
+  } else {
+    DEBUG and print "Okay, stack is empty now.\n";
+  }
+  
+  # Try generating errata section, if applicable
+  unless($self->{'~tried_gen_errata'}) {
+    $self->{'~tried_gen_errata'} = 1;
+    my @extras = $self->_gen_errata();
+    if(@extras) {
+      unshift @$paras, @extras;
+      DEBUG and print "Generated errata... relooping...\n";
+      return 1;  # I.e., loop around again to process these fake-o paragraphs
+    }
+  }
+  
+  splice @$paras; # Well, that's that for this paragraph buffer.
+  DEBUG and print "Throwing end-document event.\n";
+
+  $self->_handle_element_end( my $scratch = 'Document' );
+  return 1; # Hasta la byebye
+}
+
+sub _ponder_pod {
+  my ($self,$para,$curr_open,$paras) = @_;
+  $self->whine(
+    $para->[1]{'start_line'},
+    "=pod directives shouldn't be over one line long!  Ignoring all "
+     . (@$para - 2) . " lines of content"
+  ) if @$para > 3;
+  # Content is always ignored.
+  return;
+}
+
+sub _ponder_over {
+  my ($self,$para,$curr_open,$paras) = @_;
+  return 1 unless @$paras;
+  my $list_type;
+
+  if($paras->[0][0] eq '=item') { # most common case
+    $list_type = $self->_get_initial_item_type($paras->[0]);
+
+  } elsif($paras->[0][0] eq '=back') {
+    # Ignore empty lists.  TODO: make this an option?
+    shift @$paras;
+    return 1;
+    
+  } elsif($paras->[0][0] eq '~end') {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "=over is the last thing in the document?!"
+    );
+    return 1; # But feh, ignore it.
+  } else {
+    $list_type = 'block';
+  }
+  $para->[1]{'~type'} = $list_type;
+  push @$curr_open, $para;
+   # yes, we reuse the paragraph as a stack item
+  
+  my $content = join ' ', splice @$para, 2;
+  my $overness;
+  if($content =~ m/^\s*$/s) {
+    $para->[1]{'indent'} = 4;
+  } elsif($content =~ m/^\s*((?:\d*\.)?\d+)\s*$/s) {
+    no integer;
+    $para->[1]{'indent'} = $1;
+    if($1 == 0) {
+      $self->whine(
+        $para->[1]{'start_line'},
+        "Can't have a 0 in =over $content"
+      );
+      $para->[1]{'indent'} = 4;
+    }
+  } else {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "=over should be: '=over' or '=over positive_number'"
+    );
+    $para->[1]{'indent'} = 4;
+  }
+  DEBUG > 1 and print "=over found of type $list_type\n";
+  
+  $self->{'content_seen'} ||= 1;
+  $self->_handle_element_start((my $scratch = 'over-' . $list_type), $para->[1]);
+
+  return;
+}
+      
+sub _ponder_back {
+  my ($self,$para,$curr_open,$paras) = @_;
+  # TODO: fire off </item-number> or </item-bullet> or </item-text> ??
+
+  my $content = join ' ', splice @$para, 2;
+  if($content =~ m/\S/) {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "=back doesn't take any parameters, but you said =back $content"
+    );
+  }
+
+  if(@$curr_open and $curr_open->[-1][0] eq '=over') {
+    DEBUG > 1 and print "=back happily closes matching =over\n";
+    # Expected case: we're closing the most recently opened thing
+    #my $over = pop @$curr_open;
+    $self->{'content_seen'} ||= 1;
+    $self->_handle_element_end( my $scratch =
+      'over-' . ( (pop @$curr_open)->[1]{'~type'} )
+    );
+  } else {
+    DEBUG > 1 and print "=back found without a matching =over.  Stack: (",
+        join(', ', map $_->[0], @$curr_open), ").\n";
+    $self->whine(
+      $para->[1]{'start_line'},
+      '=back without =over'
+    );
+    return 1; # and ignore it
+  }
+}
+
+sub _ponder_item {
+  my ($self,$para,$curr_open,$paras) = @_;
+  my $over;
+  unless(@$curr_open and ($over = $curr_open->[-1])->[0] eq '=over') {
+    $self->whine(
+      $para->[1]{'start_line'},
+      "'=item' outside of any '=over'"
+    );
+    unshift @$paras,
+      ['=over', {'start_line' => $para->[1]{'start_line'}}, ''],
+      $para
+    ;
+    return 1;
+  }
+  
+  
+  my $over_type = $over->[1]{'~type'};
+  
+  if(!$over_type) {
+    # Shouldn't happen1
+    die "Typeless over in stack, starting at line "
+     . $over->[1]{'start_line'};
+
+  } elsif($over_type eq 'block') {
+    unless($curr_open->[-1][1]{'~bitched_about'}) {
+      $curr_open->[-1][1]{'~bitched_about'} = 1;
+      $self->whine(
+        $curr_open->[-1][1]{'start_line'},
+        "You can't have =items (as at line "
+        . $para->[1]{'start_line'}
+        . ") unless the first thing after the =over is an =item"
+      );
+    }
+    # Just turn it into a paragraph and reconsider it
+    $para->[0] = '~Para';
+    unshift @$paras, $para;
+    return 1;
+
+  } elsif($over_type eq 'text') {
+    my $item_type = $self->_get_item_type($para);
+      # That kills the content of the item if it's a number or bullet.
+    DEBUG and print " Item is of type ", $para->[0], " under $over_type\n";
+    
+    if($item_type eq 'text') {
+      # Nothing special needs doing for 'text'
+    } elsif($item_type eq 'number' or $item_type eq 'bullet') {
+      die "Unknown item type $item_type"
+       unless $item_type eq 'number' or $item_type eq 'bullet';
+      # Undo our clobbering:
+      push @$para, $para->[1]{'~orig_content'};
+      delete $para->[1]{'number'};
+       # Only a PROPER item-number element is allowed
+       #  to have a number attribute.
+    } else {
+      die "Unhandled item type $item_type"; # should never happen
+    }
+    
+    # =item-text thingies don't need any assimilation, it seems.
+
+  } elsif($over_type eq 'number') {
+    my $item_type = $self->_get_item_type($para);
+      # That kills the content of the item if it's a number or bullet.
+    DEBUG and print " Item is of type ", $para->[0], " under $over_type\n";
+    
+    my $expected_value = ++ $curr_open->[-1][1]{'~counter'};
+    
+    if($item_type eq 'bullet') {
+      # Hm, it's not numeric.  Correct for this.
+      $para->[1]{'number'} = $expected_value;
+      $self->whine(
+        $para->[1]{'start_line'},
+        "Expected '=item $expected_value'"
+      );
+      push @$para, $para->[1]{'~orig_content'};
+        # restore the bullet, blocking the assimilation of next para
+
+    } elsif($item_type eq 'text') {
+      # Hm, it's not numeric.  Correct for this.
+      $para->[1]{'number'} = $expected_value;
+      $self->whine(
+        $para->[1]{'start_line'},
+        "Expected '=item $expected_value'"
+      );
+      # Text content will still be there and will block next ~Para
+
+    } elsif($item_type ne 'number') {
+      die "Unknown item type $item_type"; # should never happen
+
+    } elsif($expected_value == $para->[1]{'number'}) {
+      DEBUG > 1 and print " Numeric item has the expected value of $expected_value\n";
+      
+    } else {
+      DEBUG > 1 and print " Numeric item has ", $para->[1]{'number'},
+       " instead of the expected value of $expected_value\n";
+      $self->whine(
+        $para->[1]{'start_line'},
+        "You have '=item " . $para->[1]{'number'} .
+        "' instead of the expected '=item $expected_value'"
+      );
+      $para->[1]{'number'} = $expected_value;  # correcting!!
+    }
+      
+    if(@$para == 2) {
+      # For the cases where we /didn't/ push to @$para
+      if($paras->[0][0] eq '~Para') {
+        DEBUG and print "Assimilating following ~Para content into $over_type item\n";
+        push @$para, splice @{shift @$paras},2;
+      } else {
+        DEBUG and print "Can't assimilate following ", $paras->[0][0], "\n";
+        push @$para, '';  # Just so it's not contentless
+      }
+    }
+
+
+  } elsif($over_type eq 'bullet') {
+    my $item_type = $self->_get_item_type($para);
+      # That kills the content of the item if it's a number or bullet.
+    DEBUG and print " Item is of type ", $para->[0], " under $over_type\n";
+    
+    if($item_type eq 'bullet') {
+      # as expected!
+
+      if( $para->[1]{'~_freaky_para_hack'} ) {
+        DEBUG and print "Accomodating '=item * Foo' tolerance hack.\n";
+        push @$para, delete $para->[1]{'~_freaky_para_hack'};
+      }
+
+    } elsif($item_type eq 'number') {
+      $self->whine(
+        $para->[1]{'start_line'},
+        "Expected '=item *'"
+      );
+      push @$para, $para->[1]{'~orig_content'};
+       # and block assimilation of the next paragraph
+      delete $para->[1]{'number'};
+       # Only a PROPER item-number element is allowed
+       #  to have a number attribute.
+    } elsif($item_type eq 'text') {
+      $self->whine(
+        $para->[1]{'start_line'},
+        "Expected '=item *'"
+      );
+       # But doesn't need processing.  But it'll block assimilation
+       #  of the next para.
+    } else {
+      die "Unhandled item type $item_type"; # should never happen
+    }
+
+    if(@$para == 2) {
+      # For the cases where we /didn't/ push to @$para
+      if($paras->[0][0] eq '~Para') {
+        DEBUG and print "Assimilating following ~Para content into $over_type item\n";
+        push @$para, splice @{shift @$paras},2;
+      } else {
+        DEBUG and print "Can't assimilate following ", $paras->[0][0], "\n";
+        push @$para, '';  # Just so it's not contentless
+      }
+    }
+
+  } else {
+    die "Unhandled =over type \"$over_type\"?";
+    # Shouldn't happen!
+  }
+  $para->[0] .= '-' . $over_type;
+
+  return;
+}
+
+sub _ponder_Plain {
+  my ($self,$para) = @_;
+  DEBUG and print " giving plain treatment...\n";
+  unless( @$para == 2 or ( @$para == 3 and $para->[2] eq '' )
+    or $para->[1]{'~cooked'}
+  ) {
+    push @$para,
+    @{$self->_make_treelet(
+      join("\n", splice(@$para, 2)),
+      $para->[1]{'start_line'}
+    )};
+  }
+  # Empty paragraphs don't need a treelet for any reason I can see.
+  # And precooked paragraphs already have a treelet.
+  return;
+}
+
+sub _ponder_Verbatim {
+  my ($self,$para) = @_;
+  DEBUG and print " giving verbatim treatment...\n";
+
+  $para->[1]{'xml:space'} = 'preserve';
+  for(my $i = 2; $i < @$para; $i++) {
+    foreach my $line ($para->[$i]) { # just for aliasing
+      while( $line =~
+        # Sort of adapted from Text::Tabs -- yes, it's hardwired in that
+        # tabs are at every EIGHTH column.  For portability, it has to be
+        # one setting everywhere, and 8th wins.
+        s/^([^\t]*)(\t+)/$1.(" " x ((length($2)<<3)-(length($1)&7)))/e
+      ) {}
+
+      # TODO: whinge about (or otherwise treat) unindented or overlong lines
+
+    }
+  }
+  
+  # Now the VerbatimFormatted hoodoo...
+  if( $self->{'accept_codes'} and
+      $self->{'accept_codes'}{'VerbatimFormatted'}
+  ) {
+    while(@$para > 3 and $para->[-1] !~ m/\S/) { pop @$para }
+     # Kill any number of terminal newlines
+    $self->_verbatim_format($para);
+  } elsif ($self->{'codes_in_verbatim'}) {
+    push @$para,
+    @{$self->_make_treelet(
+      join("\n", splice(@$para, 2)),
+      $para->[1]{'start_line'}, $para->[1]{'xml:space'}
+    )};
+    $para->[-1] =~ s/\n+$//s; # Kill any number of terminal newlines
+  } else {
+    push @$para, join "\n", splice(@$para, 2) if @$para > 3;
+    $para->[-1] =~ s/\n+$//s; # Kill any number of terminal newlines
+  }
+  return;
+}
+
+sub _ponder_Data {
+  my ($self,$para) = @_;
+  DEBUG and print " giving data treatment...\n";
+  $para->[1]{'xml:space'} = 'preserve';
+  push @$para, join "\n", splice(@$para, 2) if @$para > 3;
+  return;
+}
+
+
+
+
+###########################################################################
+
 sub _traverse_treelet_bit {  # for use only by the routine above
   my($self, $name) = splice @_,0,2;
 
@@ -1383,12 +1616,17 @@ sub _treelet_from_formatting_codes {
   #            "!"
   #       ]
   
-  my($self, $para, $start_line) = @_;
+  my($self, $para, $start_line, $preserve_space) = @_;
+  
   my $treelet = ['~Top', {'start_line' => $start_line},];
   
-  $para =~ s/\s+/ /g; # collapse and trim all whitespace first.
-  $para =~ s/ $//g;
-  $para =~ s/^ //g;
+  unless ($preserve_space) {
+    $para =~ s/\.  /\.\xA0 /g if $self->{'fullstop_space_harden'};
+  
+    $para =~ s/\s+/ /g; # collapse and trim all whitespace first.
+    $para =~ s/ $//;
+    $para =~ s/^ //;
+  }
   
   # Only apparent problem the above code is that N<<  >> turns into
   # N<< >>.  But then, word wrapping does that too!  So don't do that!
